/* ki2.h
 */

#ifndef _KI2_H
#define _KI2_H

#include "osl/record/record.h"
#include "osl/record/kanjiMove.h"
#include <string>
#include <iosfwd>

namespace osl
{
  namespace record
  {
    namespace ki2
    {
      class InputStream : public IRecordStream
      {
      private:
	std::istream& is;
	SimpleState state;
        osl::record::KanjiMove kmove;
	RecordVisitor rv;
      public:
	InputStream(std::istream& is, bool verbose=false);
	virtual ~InputStream();

	/** @throw Ki2IOError */
	virtual void load(Record*);
	const SimpleState& getState() const { return state;}
      };

      /**
       * 「.ki2」という拡張子を持つ2ch形式ファイル.  
       * ファイルはShift_JIS (Windows-31J)であることが期待され、
       * 内部ではEUC-JPに文字変換される。
       */
      class Ki2File
      {
      private:
        Record rec;
        bool verbose;
      public:
        Ki2File(const std::string& fileName, bool verbose=false);
        Record const& getRecord() const;
        const NumEffectState getInitialState() const;
      };

      const std::string show(Position);
      const std::string show(Position cur, Position prev);
      const std::string show(Ptype);
      const std::string showPromote(bool);
      const std::string show(Move move, const NumEffectState& state, Move prev=Move());
      const std::string show(const Move *first, const Move *last, const NumEffectState& state, Move prev=Move());
    } // namespace ki2
  } // namespace record
  using record::ki2::Ki2File;
} // namespace osl

#endif /* _KI2_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
